from .core import (
    single_quotes,
    double_quotes,
    single_quotes_stream,
    double_quotes_stream,
)

__all__ = [
    "single_quotes",
    "double_quotes",
    "single_quotes_stream",
    "double_quotes_stream",
]
